public inbox for passt-dev@passt.top
 help / color / mirror / code / Atom feed
* [PATCH] udp: Actually bind detected namespace ports in init namespace
@ 2023-03-21 15:20 Stefano Brivio
  0 siblings, 0 replies; only message in thread
From: Stefano Brivio @ 2023-03-21 15:20 UTC (permalink / raw)
  To: passt-dev; +Cc: Paul Holzinger

When I reworked udp_init() to move most of the port binding logic
to conf_ports, I accidentally dropped this bit of automatic port
detection (and binding) at start-up.

On -U auto, in pasta mode, udp_sock_init_ns() binds ports in the
namespace that correspond to ports bound in the init namespace,
but on -u auto, nothing actually happens after port detection.

Add udp_sock_init_init() to deal with this, and while at it fix
the comment to udp_sock_init_ns(): the latter takes care of
outbound "connections".

This is currently not covered by tests, and the UDP port needs to
be already bound in the namespace when pasta starts (periodic
detection for UDP is a missing feature at the moment). It can be
checked like this:

  $ unshare -rUn
  # echo $$
  590092
  # socat -u UDP-LISTEN:5555 STDOUT

  $ pasta -q -u auto 590092
  $ echo "test" | socat -u STDIN UDP:localhost:5555

Reported-by: Paul Holzinger <pholzing@redhat.com>
Fixes: 3c6ae625101a ("conf, tcp, udp: Allow address specification for forwarded ports")
Signed-off-by: Stefano Brivio <sbrivio@redhat.com>
---
 udp.c | 19 ++++++++++++++++++-
 1 file changed, 18 insertions(+), 1 deletion(-)

diff --git a/udp.c b/udp.c
index b7bc4f3..45c8d40 100644
--- a/udp.c
+++ b/udp.c
@@ -1042,7 +1042,23 @@ int udp_sock_init(const struct ctx *c, int ns, sa_family_t af,
 }
 
 /**
- * udp_sock_init_ns() - Bind sockets in namespace for inbound connections
+ * udp_sock_init_init() - Bind sockets in init namespace for inbound connections
+ * @c:		Execution context
+ */
+static void udp_sock_init_init(struct ctx *c)
+{
+	unsigned dst;
+
+	for (dst = 0; dst < NUM_PORTS; dst++) {
+		if (!bitmap_isset(c->udp.fwd_in.f.map, dst))
+			continue;
+
+		udp_sock_init(c, 0, AF_UNSPEC, NULL, NULL, dst);
+	}
+}
+
+/**
+ * udp_sock_init_ns() - Bind sockets in namespace for outbound connections
  * @arg:	Execution context
  *
  * Return: 0
@@ -1110,6 +1126,7 @@ int udp_init(struct ctx *c)
 
 	if (c->mode == MODE_PASTA) {
 		udp_splice_iov_init();
+		udp_sock_init_init(c);
 		NS_CALL(udp_sock_init_ns, c);
 	}
 
-- 
@@ -1042,7 +1042,23 @@ int udp_sock_init(const struct ctx *c, int ns, sa_family_t af,
 }
 
 /**
- * udp_sock_init_ns() - Bind sockets in namespace for inbound connections
+ * udp_sock_init_init() - Bind sockets in init namespace for inbound connections
+ * @c:		Execution context
+ */
+static void udp_sock_init_init(struct ctx *c)
+{
+	unsigned dst;
+
+	for (dst = 0; dst < NUM_PORTS; dst++) {
+		if (!bitmap_isset(c->udp.fwd_in.f.map, dst))
+			continue;
+
+		udp_sock_init(c, 0, AF_UNSPEC, NULL, NULL, dst);
+	}
+}
+
+/**
+ * udp_sock_init_ns() - Bind sockets in namespace for outbound connections
  * @arg:	Execution context
  *
  * Return: 0
@@ -1110,6 +1126,7 @@ int udp_init(struct ctx *c)
 
 	if (c->mode == MODE_PASTA) {
 		udp_splice_iov_init();
+		udp_sock_init_init(c);
 		NS_CALL(udp_sock_init_ns, c);
 	}
 
-- 
2.39.2


^ permalink raw reply related	[flat|nested] only message in thread

only message in thread, other threads:[~2023-03-21 15:20 UTC | newest]

Thread overview: (only message) (download: mbox.gz / follow: Atom feed)
-- links below jump to the message on this page --
2023-03-21 15:20 [PATCH] udp: Actually bind detected namespace ports in init namespace Stefano Brivio

Code repositories for project(s) associated with this public inbox

	https://passt.top/passt

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for IMAP folder(s).